﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Data.OleDb;

using Korzh.EasyQuery.Db;
using Korzh.EasyQuery.Services;
using Korzh.EasyQuery.Services.Db;
using System.Web.Script.Serialization;
using System.Collections;

namespace EqWebFormsDemo {
    /// <summary>
    /// Summary description for EasyQueryWebService
    /// </summary>
    [WebService(Namespace = "http://korzh.com/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    [System.Web.Script.Services.ScriptService]
    public class EasyQueryWebService : System.Web.Services.WebService {

        private EqServiceProviderDb eqService;

        public EasyQueryWebService() {
            eqService = new EqServiceProviderDb();
            eqService.DefaultModelName = "NWindMDB";

            eqService.CustomListResolver = (listname) => {
                if (listname == "Regions") {
                    return new List<ListItem> {
                        new ListItem("CA","California"),
                        new ListItem("OR","Oregon")
                    };
                }
                return Enumerable.Empty<ListItem>();
            };

            //these two linese allows eqService object access Session to store the content of model and query between requests
            eqService.SessionGetter = key => Session[key];
            eqService.SessionSetter = (key, value) => Session[key] = value;

            //we will share Query object between this service and our .aspx page - so it's better to store it in session as well
            eqService.StoreQueryInSession = true;

            //Our demo project uses Access database - so we set an appropriate format here
            eqService.Formats.SetDefaultFormats(FormatType.MsAccess);

            //Make this property true if you use SqlServer and want to include schema name into generated SQL statements
            eqService.Formats.UseSchema = false;

            //Calculating the path to App_Data folder where our model file and our database (.mdb file) are stored
            string dataPath = System.Web.HttpContext.Current.Server.MapPath("~/App_Data");
            eqService.DataPath = dataPath;

            //Setting up the connection
            eqService.Connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" +
               System.IO.Path.Combine(dataPath, "Nwind.mdb") + ";Persist Security Info=False");
            
        }

        #region auxiliary functions
        /// <summary>
        /// Converts Dictionary object to JSON
        /// </summary>
        /// <param name="data">The dictionary.</param>
        /// <returns>The correctly formatted JSON string that represents this dictionary</returns>
        private string DictionaryToJson(IDictionary<string, object> dict) {
            return new JavaScriptSerializer().Serialize(dict);
        }

        /// <summary>
        /// Converts JSON string to Dictionary object.
        /// </summary>
        /// <param name="json">A JSON string.</param>
        /// <returns>New IDictionary object created by JSON string passed in parameter</returns>
        private IDictionary<string, object> JsonToDictionary(string json) {
            return new JavaScriptSerializer().Deserialize<IDictionary<string, object>>(json);
        }

        /// <summary>
        /// Converts IEnumerable object to JSON
        /// </summary>
        /// <param name="data">The IEnumerable object.</param>
        /// <returns>The correctly formatted JSON string that represents this list</returns>
        private string ListToJson(IEnumerable list) {
            return new JavaScriptSerializer().Serialize(list);
        }

        /// <summary>
        /// Flushes the content of the string passed in parameter into response stream
        /// </summary>
        /// <param name="s"></param>
        private void FlushResult(string s) {
            Context.Response.Write(s);        
        }

        /// <summary>
        /// Flushes the content of the IDictionary object passed in parameter into response stream.
        /// Before writing to response the dictionary is serialized into JSON string
        /// </summary>
        /// <param name="dict">The IDictionary object</param>
        private void FlushResult(IDictionary<string, object> dict) {
            Context.Response.Write(DictionaryToJson(dict));
        }

        /// <summary>
        /// Flushes the content of the IEnumerable object passed in parameter into response stream.
        /// Before writing to response the list is serialized into JSON string
        /// </summary>
        /// <param name="list">The IEnumerable object</param>
        private void FlushResult(IEnumerable list) {
            Context.Response.Write(ListToJson(list));
        }

        #endregion


        /// <summary>
        /// Gets the model by its name
        /// </summary>
        /// <param name="modelName">The name of requested model. 
        /// If null or empty this method returns current model object stored in Session
        /// </param>
        /// <returns></returns>
        [WebMethod(EnableSession=true)]
        public void GetModel(string modelName) {
            var model = eqService.GetModel(modelName);
            FlushResult(model.SaveToDictionary());
        }

        /// <summary>
        /// Gets the query by its name
        /// </summary>
        /// <param name="queryName">The name of the query. If null or empty this method returns current query object stored in Session</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true)]
        public void GetQuery(string queryName) {
            var query = eqService.GetQuery(queryName);
            FlushResult(query.SaveToDictionary());
        }


        /// <summary>
        /// It's called when it's necessary to synchronize query on client side with its server-side copy.
        /// Additionally this action can be used to return a generated SQL statement (or several statements) as JSON string
        /// </summary>
        /// <param name="queryJson">The JSON representation of the query .</param>
        /// <param name="optionsJson">The additional parameters which can be passed to this method to adjust query statement generation.</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true)]
        public void SyncQuery(string queryJson, string optionsJson) {
            IDictionary<string, object> queryDict = JsonToDictionary(queryJson);
            eqService.SyncQueryDict(queryDict);

            DbQuery query = eqService.LoadQueryDict(queryDict);
            var statement = eqService.BuildQuery(query, JsonToDictionary(optionsJson));
            Dictionary<string, object> dict = new Dictionary<string, object>();
            dict.Add("statement", statement);
            FlushResult(dict);
        }


        /// <summary>
        /// Saves the query.
        /// </summary>
        /// <param name="queryJson">The JSON representation of the query .</param>
        /// <param name="queryName">Query name.</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true)]
        public void SaveQuery(string queryJson, string queryName) {
            eqService.SaveQueryDict(JsonToDictionary(queryJson), queryName);
            Dictionary<string, object> dict = new Dictionary<string, object>();
            dict.Add("result", "OK");
            FlushResult(dict);
        }

        /// <summary>
        /// This action returns a custom list by list name and/or SQL statement (for SQLLIST value editors).
        /// </summary>
        /// <param name="options">List request options - an instance of <see cref="ListRequestOptions"/> type.</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true)]
        public void GetList(string listName, string sql) {
            ListRequestOptions options = new ListRequestOptions() { listName = listName, sql = sql };
            FlushResult(ListToJson(eqService.GetList(options)));
        }


        /// <summary>
        /// Executes the query passed as JSON string and returns the result record set (again as JSON).
        /// </summary>
        /// <param name="queryJson">The JSON representation of the query.</param>
        /// <param name="optionsJson">Different options in JSON format</param>
        /// <returns></returns>
        [WebMethod(EnableSession = true)]
        public void ExecuteQuery(string queryJson, string optionsJson) {
            var query = eqService.LoadQueryDict(JsonToDictionary(queryJson));
            var sql = eqService.BuildQuery(query, JsonToDictionary(optionsJson));

            var resultSet = eqService.GetDataSetBySql(sql);
            Session["EQ_LAST_RESULT_SET"] = resultSet;

            var resultSetDict = eqService.DataSetToDictionary(resultSet, JsonToDictionary(optionsJson));

            Dictionary<string, object> dict = new Dictionary<string, object>();
            dict.Add("statement", sql);
            dict.Add("resultSet", resultSetDict);

            FlushResult(dict);
        }

    }

}
